@@ -111,6 +111,7 @@ struct _GladeParseState {
     enum {PROP_NONE, PROP_WIDGET, PROP_ATK, PROP_CHILD } prop_type;
     gchar *prop_name;
     gboolean translate_prop;
+    gboolean context_prop;
     GArray *props;
 
     GArray *signals;
@@ -159,13 +160,13 @@ create_widget_info(GladeInterface *interface, const xmlChar **attrs)
 
     for (i = 0; attrs && attrs[i] != NULL; i += 2) {
 	if (!strcmp(attrs[i], "class"))
-	    info->class = alloc_string(interface, attrs[i+1]);
+	    info->classname = alloc_string(interface, attrs[i+1]);
 	else if (!strcmp(attrs[i], "id"))
 	    info->name = alloc_string(interface, attrs[i+1]);
 	else
 	    g_warning("unknown attribute `%s' for <widget>.", attrs[i]);
     }
-    if (info->class == NULL || info->name == NULL)
+    if (info->classname == NULL || info->name == NULL)
 	g_warning("<widget> element missing required attributes!");
     g_hash_table_insert(interface->names, info->name, info);
     return info;
@@ -261,7 +262,7 @@ static inline void
 handle_atk_action(GladeParseState *state, const xmlChar **attrs)
 {
     gint i;
-    GladeAtkActionInfo info = { 0 };
+    GladeAtkActionInfo info = { NULL };
 
     flush_properties(state);
 
@@ -269,7 +270,8 @@ handle_atk_action(GladeParseState *state, const xmlChar **attrs)
 	if (!strcmp(attrs[i], "action_name"))
 	    info.action_name = alloc_string(state->interface, attrs[i+1]);
 	else if (!strcmp(attrs[i], "description"))
-	    info.description = alloc_string(state->interface, attrs[i+1]);
+	    info.description = alloc_string(state->interface,
+				    dgettext(state->domain, attrs[i+1]));
 	else
 	    g_warning("unknown attribute `%s' for <action>.", attrs[i]);
     }
@@ -287,7 +289,7 @@ static inline void
 handle_atk_relation(GladeParseState *state, const xmlChar **attrs)
 {
     gint i;
-    GladeAtkRelationInfo info = { 0 };
+    GladeAtkRelationInfo info = { NULL };
 
     flush_properties(state);
 
@@ -312,7 +314,7 @@ handle_atk_relation(GladeParseState *state, const xmlChar **attrs)
 static inline void
 handle_signal(GladeParseState *state, const xmlChar **attrs)
 {
-    GladeSignalInfo info = { 0 };
+    GladeSignalInfo info = { NULL };
     gint i;
 
     flush_properties(state);
@@ -586,8 +588,12 @@ glade_parser_start_element(GladeParseState *state,
 						      attrs[i+1]);
 		else if (!strcmp(attrs[i], "translatable"))
 		    state->translate_prop = !strcmp(attrs[i+1], "yes");
+		else if (!strcmp(attrs[i], "context"))
+		    state->context_prop = !strcmp(attrs[i+1], "yes");
 		else if (!strcmp(attrs[i], "agent"))
 		    bad_agent = strcmp(attrs[i], "libglade") != 0;
+		else if (!strcmp(attrs[i], "comments"))
+		    /* Do nothing. */;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
@@ -642,6 +648,10 @@ glade_parser_start_element(GladeParseState *state,
 						      attrs[i+1]);
 		else if (!strcmp(attrs[i], "translatable"))
 		    state->translate_prop = !strcmp(attrs[i+1], "yes");
+		else if (!strcmp(attrs[i], "context"))
+		    state->context_prop = !strcmp(attrs[i+1], "yes");
+		else if (!strcmp(attrs[i], "comments"))
+		    /* Do nothing. */;
 		else
 		    g_warning("unknown attribute `%s' for <atkproperty>.",
 			      attrs[i]);
@@ -791,8 +801,12 @@ glade_parser_start_element(GladeParseState *state,
 						      attrs[i+1]);
 		else if (!strcmp(attrs[i], "translatable"))
 		    state->translate_prop = !strcmp(attrs[i+1], "yes");
+		else if (!strcmp(attrs[i], "context"))
+		    state->context_prop = !strcmp(attrs[i+1], "yes");
 		else if (!strcmp(attrs[i], "agent"))
 		    bad_agent = strcmp(attrs[i], "libglade") != 0;
+		else if (!strcmp(attrs[i], "comments"))
+		    /* Do nothing. */;
 		else
 		    g_warning("unknown attribute `%s' for <property>.",
 			      attrs[i]);
@@ -899,12 +913,17 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop && state->content->str[0] != '\0') {
-	    prop.value = alloc_string(state->interface,
-			dgettext(state->domain, state->content->str));
-	} else {
-	    prop.value = alloc_string(state->interface, state->content->str);
-	}
+ 	if (state->translate_prop && state->content->str[0] != '\0') {
+	    if (state->context_prop) 
+		prop.value = alloc_string(state->interface,
+					  g_strip_context(state->content->str,
+							   dgettext(state->domain, state->content->str)));
+	    else
+		prop.value = alloc_string(state->interface,
+					  dgettext(state->domain, state->content->str));
+ 	} else {
+ 	    prop.value = alloc_string(state->interface, state->content->str);
+  	}
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
 	state->state = PARSER_WIDGET;
@@ -921,12 +940,17 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop && state->content->str[0] != '\0') {
-	    prop.value = alloc_string(state->interface,
-			dgettext(state->domain, state->content->str));
-	} else {
-	    prop.value = alloc_string(state->interface, state->content->str);
-	}
+ 	if (state->translate_prop && state->content->str[0] != '\0') {
+	    if (state->context_prop) 
+		prop.value = alloc_string(state->interface,
+					  g_strip_context(state->content->str,
+							   dgettext(state->domain, state->content->str)));
+	    else
+		prop.value = alloc_string(state->interface,
+					  dgettext(state->domain, state->content->str));
+ 	} else {
+ 	    prop.value = alloc_string(state->interface, state->content->str);
+  	}
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
 	state->state = PARSER_WIDGET_ATK;
@@ -983,12 +1007,17 @@ glade_parser_end_element(GladeParseState *state, const xmlChar *name)
 	if (!state->props)
 	    state->props = g_array_new(FALSE, FALSE, sizeof(GladeProperty));
 	prop.name = state->prop_name;
-	if (state->translate_prop && state->content->str[0] != '\0') {
-	    prop.value = alloc_string(state->interface,
-			dgettext(state->domain, state->content->str));
-	} else {
-	    prop.value = alloc_string(state->interface, state->content->str);
-	}
+ 	if (state->translate_prop && state->content->str[0] != '\0') {
+	    if (state->context_prop) 
+		prop.value = alloc_string(state->interface,
+					  g_strip_context(state->content->str,
+							   dgettext(state->domain, state->content->str)));
+	    else
+		prop.value = alloc_string(state->interface,
+					  dgettext(state->domain, state->content->str));
+ 	} else {
+ 	    prop.value = alloc_string(state->interface, state->content->str);
+  	}
 	g_array_append_val(state->props, prop);
 	state->prop_name = NULL;
 	state->state = PARSER_WIDGET_CHILD_PACKING;
@@ -1071,30 +1100,30 @@ glade_parser_fatal_error(GladeParseState *state, const char *msg, ...)
 }
 
 static xmlSAXHandler glade_parser = {
-    0, /* internalSubset */
-    0, /* isStandalone */
-    0, /* hasInternalSubset */
-    0, /* hasExternalSubset */
-    0, /* resolveEntity */
-    (getEntitySAXFunc)glade_parser_get_entity, /* getEntity */
-    0, /* entityDecl */
-    0, /* notationDecl */
-    0, /* attributeDecl */
-    0, /* elementDecl */
-    0, /* unparsedEntityDecl */
-    0, /* setDocumentLocator */
+    (internalSubsetSAXFunc)NULL,                       /* internalSubset */
+    (isStandaloneSAXFunc)NULL,                         /* isStandalone */
+    (hasInternalSubsetSAXFunc)NULL,                    /* hasInternalSubset */
+    (hasExternalSubsetSAXFunc)NULL,                    /* hasExternalSubset */
+    (resolveEntitySAXFunc)NULL,                        /* resolveEntity */
+    (getEntitySAXFunc)glade_parser_get_entity,         /* getEntity */
+    (entityDeclSAXFunc)NULL,                           /* entityDecl */
+    (notationDeclSAXFunc)NULL,                         /* notationDecl */
+    (attributeDeclSAXFunc)NULL,                        /* attributeDecl */
+    (elementDeclSAXFunc)NULL,                          /* elementDecl */
+    (unparsedEntityDeclSAXFunc)NULL,                   /* unparsedEntityDecl */
+    (setDocumentLocatorSAXFunc)NULL,                   /* setDocumentLocator */
     (startDocumentSAXFunc)glade_parser_start_document, /* startDocument */
-    (endDocumentSAXFunc)glade_parser_end_document, /* endDocument */
-    (startElementSAXFunc)glade_parser_start_element, /* startElement */
-    (endElementSAXFunc)glade_parser_end_element, /* endElement */
-    0, /* reference */
-    (charactersSAXFunc)glade_parser_characters, /* characters */
-    0, /* ignorableWhitespace */
-    0, /* processingInstruction */
-    (commentSAXFunc)0, /* comment */
-    (warningSAXFunc)glade_parser_warning, /* warning */
-    (errorSAXFunc)glade_parser_error, /* error */
-    (fatalErrorSAXFunc)glade_parser_fatal_error, /* fatalError */
+    (endDocumentSAXFunc)glade_parser_end_document,     /* endDocument */
+    (startElementSAXFunc)glade_parser_start_element,   /* startElement */
+    (endElementSAXFunc)glade_parser_end_element,       /* endElement */
+    (referenceSAXFunc)NULL,                            /* reference */
+    (charactersSAXFunc)glade_parser_characters,        /* characters */
+    (ignorableWhitespaceSAXFunc)NULL,               /* ignorableWhitespace */
+    (processingInstructionSAXFunc)NULL,             /* processingInstruction */
+    (commentSAXFunc)NULL,                              /* comment */
+    (warningSAXFunc)glade_parser_warning,              /* warning */
+    (errorSAXFunc)glade_parser_error,                  /* error */
+    (fatalErrorSAXFunc)glade_parser_fatal_error,       /* fatalError */
 };
 
 static void
@@ -1107,6 +1136,8 @@ widget_info_free(GladeWidgetInfo *info)
     g_free(info->properties);
     g_free(info->atk_props);
     g_free(info->signals);
+    g_free(info->atk_actions);
+    g_free(info->relations);
     g_free(info->accels);
 
     for (i = 0; i < info->n_children; i++) {
@@ -1165,6 +1196,11 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
 {
     GladeParseState state = { 0 };
 
+    if (!g_file_test(file, G_FILE_TEST_IS_REGULAR)) {
+	g_warning("could not find glade file '%s'", file);
+	return NULL;
+    }
+
     state.interface = NULL;
     if (domain)
 	state.domain = domain;
@@ -1172,13 +1208,13 @@ glade_parser_parse_file(const gchar *file, const gchar *domain)
 	state.domain = textdomain(NULL);
 
     if (xmlSAXUserParseFile(&glade_parser, &state, file) < 0) {
-	g_warning("document not well formed!");
+	g_warning("document not well formed");
 	if (state.interface)
 	    glade_interface_destroy (state.interface);
 	return NULL;
     }
     if (state.state != PARSER_FINISH) {
-	g_warning("did not finish in PARSER_FINISH state!");
+	g_warning("did not finish in PARSER_FINISH state");
 	if (state.interface)
 	    glade_interface_destroy(state.interface);
 	return NULL;
@@ -1233,7 +1269,7 @@ dump_widget(xmlNode *parent, GladeWidgetInfo *info, gint indent)
     xmlNode *widget = xmlNewNode(NULL, "widget");
     gint i, j;
 
-    xmlSetProp(widget, "class", info->class);
+    xmlSetProp(widget, "class", info->classname);
     xmlSetProp(widget, "id", info->name);
     xmlAddChild(parent, widget);
     xmlNodeAddContent(widget, "\n");
